﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace COA.EnterpriseServices.DataAccess.Helpers
{
    public class BaseHelper<T> where T : class, IRecord
    {
        protected readonly ICollection<IDataAccess<T>> dataAccessInstances;

        public BaseHelper()
        {
            dataAccessInstances = Dependencies.Container
                .GetAllInstances<IDataAccess<T>>()
                .OrderBy(i => i.GetType().Name.StartsWith("QuickBase", StringComparison.OrdinalIgnoreCase))
                .ToList();
        }

        protected TResult Invoke<TResult>(Func<IDataAccess<T>, TResult> func)
        {
            var results = new List<TResult>();

            foreach (var dataAccessInstance in dataAccessInstances)
            {
                results.Add(func(dataAccessInstance));
            }

            // if the "EF" version invokes first, return that value
            return results.FirstOrDefault(r => r != null && !r.Equals(default(TResult)));
        }
    }
}
